#version 430 core

layout(points) in; // points in
layout (triangle_strip) out;
layout (max_vertices = 4) out;     // quads out

uniform mat4 ProjectionMatrix = 
 mat4
(1, 0, 0, 0
, 0, 1, 0, 0,
0, 0, 1, 0,
0, 0, 0, 1); 

in vData{ 
	vec3 position_world;
	float TTL1;
	vec3 currVelocity;
	float particleSize;
} vertex[]; // from vertex shader


uniform float maxTTL = 10.f;
uniform float startPercentSize = 0.1f;
uniform float endPercentSize = 0.1f;
uniform vec2 particleBaseSize = vec2(5, 5);


flat out float TTL0; // flat: no interpolation between vertices


// Cell shading
 out vertexData{
	flat vec3 position_world;
	vec2 uv;
} vert;


// Prototypes
vec2 calcCurrentSize(float currTTL);


void main (void) {
	vec2 size = vec2(5, 5);
	float currTTL = vertex[0].TTL1;

	size = calcCurrentSize(currTTL);


	vec4 P = gl_in[0].gl_Position; // position in view-space
	vec4 tempPosition;

	vec2 v = P.xy + vec2(0.5, 0.5) * size; // top-right corner
	
	tempPosition = ProjectionMatrix * vec4(v, P.zw);

	// vert.position_world = tempPosition.xyz;
	vert.position_world = (ProjectionMatrix * P).xyz;
		
	gl_Position= tempPosition;
	vert.uv = vec2(1.0, 1.0);
	TTL0 = currTTL;
	EmitVertex();

	v = P.xy + vec2(-0.5, 0.5) * size; // top-left corner
	
	tempPosition = ProjectionMatrix * vec4(v, P.zw);

	// vert.position_world = tempPosition.xyz;
	vert.position_world = (ProjectionMatrix * P).xyz;
		
	gl_Position= tempPosition;
	vert.uv = vec2(0.0, 1.0);
	TTL0 = currTTL;
	EmitVertex();

	v = P.xy + vec2(0.5, -0.5) * size; // bottom-right corner
	
	tempPosition = ProjectionMatrix * vec4(v, P.zw);

	// vert.position_world = tempPosition.xyz;
	vert.position_world = (ProjectionMatrix * P).xyz;
		
	gl_Position= tempPosition;
	vert.uv = vec2(1.0, 0.0);
	TTL0 = currTTL;
	EmitVertex();

	v = P.xy + vec2(-0.5, -0.5) * size; // bottom-left corner
	
	tempPosition = ProjectionMatrix * vec4(v, P.zw);

	// vert.position_world = tempPosition.xyz;
	vert.position_world = (ProjectionMatrix * P).xyz;
			
	gl_Position= tempPosition;
	vert.uv = vec2(0.0, 0.0);
	TTL0 = currTTL;
	EmitVertex();

	EndPrimitive();
}

vec2 calcCurrentSize(float currTTL)
{
	float halfTime = maxTTL / 2;
	float partSize = vertex[0].particleSize;
	vec2 size = (partSize < 0.001) ? particleBaseSize : vec2(partSize, partSize);

	if(currTTL > halfTime)
	{
		float currStep = (currTTL - halfTime) / halfTime;

		return size * mix(1, startPercentSize, currStep); 
	}
	else if (currTTL == halfTime)
	{
		return size;
	}
	else
	{
		float currStep = (halfTime - currTTL) / halfTime;
		return size * mix(1, endPercentSize, currStep); 
	}
}