#version 430 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 normalsTexture;

uniform vec3 camera_world;

uniform float currentTime;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
//uniform sampler2D diffuseTexture;
uniform sampler1D cellShadeTexture;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


float cellShadeLightIntensity(vec3 normallizedNormal, vec3 lightDir, vec3 viewDir, float diffuseF, float specularF, float alpha, bool attenuate, vec3 attenuation, vec3 lightCol) {
	float d = length(lightDir);
	lightDir = normalize(lightDir);
	float lightColIntensity = length(lightCol);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 lightRef = reflect(-lightDir, normallizedNormal);
	return (diffuseF * max(0, dot(normallizedNormal, lightDir)) *att + specularF * pow(max(0, dot(lightRef, viewDir)), alpha) * att) * lightColIntensity; 
}

//Temp Noise from OpenGl Insights

// Permutation polynomial: (34x^2 + 5x) mod 289
vec4 permute(vec4 x) {
  return mod((34.0 * x + 5.0) * x, 289.0);
}

// Cellular noise, returning F1 and F2 in a vec2.
// Speeded up by using 2x2 search window instead of 3x3,
// at the expense of some strong pattern artifacts.
// F2 is often wrong and has sharp discontinuities.
// If you need a smooth F2, use the slower 3x3 version.
// F1 is sometimes wrong, too, but OK for most purposes.
vec2 cellular2x2(vec2 P) {
#define K 0.142857142857 // 1/7
#define K2 0.0714285714285 // K/2
#define jitter 1.0 // jitter 1.0 makes F1 wrong more often
	vec2 Pi = mod(floor(P), 289.0);
 	vec2 Pf = fract(P);
	vec4 Pfx = Pf.x + vec4(-0.5, -1.5, -0.5, -1.5);
	vec4 Pfy = Pf.y + vec4(-0.5, -0.5, -1.5, -1.5);
	vec4 p = permute(Pi.x + vec4(0.0, 1.0, 0.0, 1.0));
	p = permute(p + Pi.y + vec4(0.0, 0.0, 1.0, 1.0));
	vec4 ox = mod(p, 7.0)*K+K2;
	vec4 oy = mod(floor(p*K),7.0)*K+K2;
	vec4 dx = Pfx + jitter*ox;
	vec4 dy = Pfy + jitter*oy;
	vec4 d = dx * dx + dy * dy; // d11, d12, d21 and d22, squared
	// Sort out the two smallest distances
#if 0
	// Cheat and pick only F1
	d.xy = min(d.xy, d.zw);
	d.x = min(d.x, d.y);
	return d.xx; // F1 duplicated, F2 not computed
#else
	// Do it right and find both F1 and F2
	d.xy = (d.x < d.y) ? d.xy : d.yx; // Swap if smaller
	d.xz = (d.x < d.z) ? d.xz : d.zx;
	d.xw = (d.x < d.w) ? d.xw : d.wx;
	d.y = min(d.y, d.z);
	d.y = min(d.y, d.w);
	return sqrt(d.xy);
#endif
}

//Temp Noise from OpenGl Insights

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	

	vec3 texColor = vec3 (cellular2x2(vert.uv),1).rgb;
	
	
	//vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	color = vec4(texColor * materialCoefficients.x, 1); // ambient
	
	
	// cellShadeLightIntensity(vec3 normallizedNormal, vec3 lightDir, vec3 viewDir, float diffuseF, float specularF, float alpha, bool attenuate, vec3 attenuation)
	float intensity = 0;
	intensity += cellShadeLightIntensity(n, -dirL.direction, v, materialCoefficients.y, materialCoefficients.z, specularAlpha, false, vec3(0), dirL.color);
	intensity += cellShadeLightIntensity(n, pointL.position - vert.position_world, v, materialCoefficients.y, materialCoefficients.z, specularAlpha, true, pointL.attenuation, pointL.color);	

	if(intensity < 1)
	{
		intensity = texture(cellShadeTexture, intensity).r;
	}
	else
	{
		intensity = 1.;
	}
	color.rgb += texColor * intensity *2;
	normalsTexture.rgb = vert.normal_world;

}

