#version 430 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 normalsTexture;

uniform vec3 camera_world;

uniform float currentTime1;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler1D cellShadeTexture;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


float cellShadeLightIntensity(vec3 normallizedNormal, vec3 lightDir, vec3 viewDir, float diffuseF, float specularF, float alpha, bool attenuate, vec3 attenuation, vec3 lightCol) {
	float d = length(lightDir);
	lightDir = normalize(lightDir);
	float lightColIntensity = length(lightCol);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 lightRef = reflect(-lightDir, normallizedNormal);
	return (diffuseF * max(0, dot(normallizedNormal, lightDir)) *att + specularF * pow(max(0, dot(lightRef, viewDir)), alpha) * att) * lightColIntensity; 
}

//Randomize vec2
vec2 random2( vec2 p ) {
    return fract(sin(vec2(dot(p,vec2(127.1,311.7)),dot(p,vec2(269.5,183.3))))*43758.5453);
}


vec3 Noise(vec2 uvs) {
	
	//Strech because Water.obj is very long
    vec2 resolution = vec2(160,3200);
    vec2 st = uvs.xy * resolution.xy;

	//Parameters
		//Choice of Color 
		//vec3 color = vec3(0.003,0.084,0.185);
		vec3 color = vec3(0.,0.,0.);
	  
		// Scale
		st *= 0.1;
		//Minimum Distance 
		float m_dist = 1.;

    vec2 i_st = floor(st);
    vec2 f_st = fract(st);

    

    for (int y= -1; y <= 1; y++) {
        for (int x= -1; x <= 1; x++) {
			
			//Move Points
            vec2 neighbor = vec2(float(x),float(y));
            vec2 point = random2(i_st + neighbor);
            point = 0.5 + 0.5*sin(currentTime1 + 6.2831*point);

			//Get distance
            vec2 diff = neighbor + point - f_st;
            float dist = length(diff);
            m_dist = min(m_dist, dist);
        }
    }

    // Draw minimum Distance
    color += m_dist;

    return color;
}



void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = Noise(vert.uv);

//	vec3 texColor = vec3(1.f);
	color = vec4(texColor * materialCoefficients.x, 1); // ambient
	
	
	// cellShadeLightIntensity(vec3 normallizedNormal, vec3 lightDir, vec3 viewDir, float diffuseF, float specularF, float alpha, bool attenuate, vec3 attenuation)
	float intensity = 0;
	intensity += cellShadeLightIntensity(n, -dirL.direction, v, materialCoefficients.y, materialCoefficients.z, specularAlpha, false, vec3(0), dirL.color);
	intensity += cellShadeLightIntensity(n, pointL.position - vert.position_world, v, materialCoefficients.y, materialCoefficients.z, specularAlpha, true, pointL.attenuation, pointL.color);	

	if(intensity < 1)
	{
		intensity = texture(cellShadeTexture, intensity).r;
	}
	else
	{
		intensity = 1.;
	}
	color.rgb += texColor * intensity ;
	normalsTexture.rgb = vert.normal_world;

}

