#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

uniform float currentTime = 0.f;
uniform float amplitude = 0.6f;
uniform float frequency = 1.9f;

void main() {
	//vert.normal_world = normalMatrix * normal;
	vert.uv = uv;
	vec4 worldPos = modelMatrix * vec4(position, 1);
	
	

	//wave
	worldPos.y += sin(worldPos.x * frequency + currentTime) * amplitude + sin(worldPos.z * frequency + currentTime) * amplitude;
	//spike
	//worldPos.y += sin(currentTime * frequency + worldPos.x + mod(worldPos.z,2) ) * sin(mod(worldPos.z,2) + currentTime  * frequency) * amplitude;
	vert.position_world = worldPos.xyz;
	gl_Position = viewProjMatrix * worldPos;
	
		//wave
	vec3 normaldx = vec3 (1, frequency * amplitude * (cos(worldPos.x * frequency + currentTime)),0);
	vec3 normaldz = vec3 (0, frequency * amplitude * (cos(worldPos.z * frequency + currentTime)),1);
	//spike 
	//vec3 normaldx = vec3 (1,  -(cos(currentTime + worldPos.x + mod(worldPos.z,2) * frequency)* sin(mod(worldPos.z,2) + currentTime * frequency) * amplitude),0);
	//vec3 normaldz = vec3 (0,  (sin(currentTime + worldPos.x + mod(worldPos.z,2) * frequency) * cos(mod(worldPos.z,2) + currentTime * frequency) * amplitude + cos(currentTime + worldPos.x + mod(worldPos.z,2) * frequency) * sin(mod(worldPos.z,2) + currentTime * frequency) * amplitude) ,1);
	vec3 temp =  cross(normaldx,normaldz);
	temp = normalize(temp);
	temp.y = abs(temp.y);
	vert.normal_world =  temp;
	
}