#version 430 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 normalsTexture;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler1D cellShadeTexture;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


float cellShadeLightIntensity(vec3 normallizedNormal, vec3 lightDir, vec3 viewDir, float diffuseF, float specularF, float alpha, bool attenuate, vec3 attenuation, vec3 lightCol) {
	float d = length(lightDir);
	lightDir = normalize(lightDir);
	float lightColIntensity = length(lightCol);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 lightRef = reflect(-lightDir, normallizedNormal);
	return (diffuseF * max(0, dot(normallizedNormal, lightDir)) *att + specularF * pow(max(0, dot(lightRef, viewDir)), alpha) * att) * lightColIntensity; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	color = vec4(texColor * materialCoefficients.x, 1); // ambient
	
	
	// cellShadeLightIntensity(vec3 normallizedNormal, vec3 lightDir, vec3 viewDir, float diffuseF, float specularF, float alpha, bool attenuate, vec3 attenuation)
	float intensity = 0;
	intensity += cellShadeLightIntensity(n, -dirL.direction, v, materialCoefficients.y, materialCoefficients.z, specularAlpha, false, vec3(0), dirL.color);
	intensity += cellShadeLightIntensity(n, pointL.position - vert.position_world, v, materialCoefficients.y, materialCoefficients.z, specularAlpha, true, pointL.attenuation, pointL.color);	

	if(intensity < 1)
	{
		intensity = texture(cellShadeTexture, intensity).r;
	}
	else
	{
		intensity = 1.;
	}
	color.rgb += texColor * intensity *2;
	normalsTexture.rgb = vert.normal_world;

}

